#!/usr/bin/env Rscript

## To load in interactive R shell, run: source('classifier-analysis.R')

## install.packages(c("broom", "cowplot", "dplyr", "ggplot2", "scales", "tidyr", "xtable")

## Useful web sites: http://r4ds.had.co.nz/ , https://www.tidyverse.org/
##
library('broom')
# library('cowplot')
library('dplyr')
library('ggplot2')
library('scales')
library('tidyr')
library('xtable')

## https://stackoverflow.com/questions/5831794/opposite-of-in
`%not_in%` = function (x, table) is.na(match(x, table, nomatch=NA_integer_))  

raw_data = bind_rows('0'=read.csv('classifier-results.csv', header=T),
                     '1'=read.csv('shuffle-results1.csv', header=T),
                     '2'=read.csv('shuffle-results2.csv', header=T),
                     '3'=read.csv('shuffle-results3.csv', header=T),
                     '4'=read.csv('shuffle-results4.csv', header=T),
                     '5'=read.csv('shuffle-results5.csv', header=T),
                     '6'=read.csv('shuffle-results6.csv', header=T),
                     '7'=read.csv('shuffle-results7.csv', header=T),
                     '8'=read.csv('shuffle-results0.csv', header=T),
                     .id='Run') %>% mutate(Run=factor(Run))


raw_data = raw_data %>%
    filter(complete.cases(.)) %>%              # Remove NAs
    mutate(error=1.0-prec,                     # Calculate error
           Training.file=training_file,
           training.examples=size,
           config=factor(config)) %>%          # Convert config to factor
    select(-prec,-rec,-size, -training_file)   # Remove unnecessary columns


data = raw_data %>%
    group_by(config, Training.file, training.examples) %>%
    summarise(av.error=mean(error)) %>%
    group_by(Training.file, training.examples) %>%
    filter(rank(av.error, ties.method="random") == 1) %>%
    mutate(min.av.error=av.error) %>%
    select(Training.file, training.examples, min.av.error) %>%
    left_join(raw_data) %>%
    mutate(rel.error=error/min.av.error)

## qplot(x=factor(lr), y=rel.error, data=data, geom="violin", log="y")
## qplot(x=factor(dim), y=rel.error, data=data, geom="violin", log="y")
## qplot(x=factor(loss), y=rel.error, data=data, geom="violin", log="y")
## qplot(x=factor(wordNgrams), y=rel.error, data=data, geom="violin", log="y")

cscores = data %>%
    group_by(config, lr, dim, loss, wordNgrams) %>%
    summarise(max.rel.error=max(rel.error),
              min.rel.error=min(rel.error),
              mean.rel.error=mean(rel.error),
              sd.rel.error=sd(rel.error),
              q10.rel.error=quantile(rel.error,probs=c(0.1)),
              q25.rel.error=quantile(rel.error,probs=c(0.25)),
              q50.rel.error=quantile(rel.error,probs=c(0.5)),
              q75.rel.error=quantile(rel.error,probs=c(0.75)),
              q90.rel.error=quantile(rel.error,probs=c(0.9))) %>%
    arrange(max.rel.error)

summary(cscores %>% filter(max.rel.error<1.165))
