#!/usr/bin/env python3

"""
Compute summary statistics for fastText data files.
"""

import argparse
import csv
import sys

def parse_args():
  """Parse command-line arguments"""
  p = argparse.ArgumentParser(description=__doc__,
                              formatter_class=argparse.RawDescriptionHelpFormatter)
  p.add_argument('--output', '-o', type=argparse.FileType('w'), default=sys.stdout,
                 help='CSV stats file')
  p.add_argument('fastText_files', nargs='+', 
                 help="fastText data files to summarise")
  return p.parse_args()

csvheader = ['Training.file', 'type', 'training.examples', 'nlabels']

def file_stats(fname):
  """Returns statistics of fastText data file in fname"""
  nlines = 0
  labels = set()
  with open(fname, 'rU') as inf:
    for line in inf:
      label = line[:line.find(' ')]
      labels.add(label)
      nlines += 1
  basename = fname.split('/')[-1].rsplit('.', maxsplit=1)
  return (basename[0], basename[1], nlines, len(labels))

if __name__ == '__main__':
  args = parse_args()
  outf = csv.writer(args.output)
  outf.writerow(csvheader)
  args.output.flush()
  for fname in args.fastText_files:
    outf.writerow(file_stats(fname))
    args.output.flush()
