#!/usr/bin/env R

## To load in interactive R shell, run: source('hyperparams.R')

## install.packages(c("broom", "cowplot", "dplyr", "ggplot2", "scales", "tidyr", "xtable")

## Useful web sites: http://dplyr.tidyverse.org/articles/programming.html , http://r4ds.had.co.nz/ , https://www.tidyverse.org/
##
library('broom')
# library('cowplot')
library('dplyr')
library('ggplot2')
library('scales')
library('tidyr')
library('xtable')

test_file_names = c("amazon_review_polarity","sogou_news","yahoo_answers","yelp_review_full")

## https://stackoverflow.com/questions/5831794/opposite-of-in
`%not_in%` = function (x, table) is.na(match(x, table, nomatch=NA_integer_))  

options(dplyr.width = 100)  # Number of columns to print


# $ scp rakili:research/git/extrapolate/fastText/classifier-results.csv .
raw_data = read.csv('classifier-results.csv', header=T)
raw_data = raw_data %>%
    filter(complete.cases(.)) %>%              # Remove NAs
    mutate(error=1.0-prec,                     # Calculate error
           Training.file=factor(training_file),
           dim=factor(dim),
           training.examples=size,
           config=factor(config)) %>%          # Convert config to factor
    select(-prec,-rec,-size, -training_file)   # Remove unnecessary columns


min_error = raw_data %>%
    group_by(Training.file, training.examples) %>%
    slice(which.min(error)) %>%
    mutate(min.error=error) %>%
    select(-config,-error)

min_error %>%
    select(-dim) %>%
    left_join(raw_data) %>%
    group_by('Training.file') %>%
    mutate(rel.error=pmin(2.0, error/min.error)) %>%
    ggplot(mapping=aes(dim,factor(training.examples))) +
        geom_raster(aes(fill=rel.error)) +
        scale_fill_gradient(low = "white", high = "steelblue") +
        facet_wrap('Training.file', scales="free")

hmap = function (data, attr, max_rel_error=1.5) {
    attr = enquo(attr)
    data = data %>% mutate(training.examples=factor(training.examples), lr=factor(lr))
    levels(data$training.examples) = sprintf("%4.2g", as.numeric(levels(data$training.examples)))
    data %>%
        group_by(Training.file, training.examples) %>%
        slice(which.min(error)) %>%
        mutate(min.error=error) %>%
        select(-config,-error,- !!attr) %>%
        left_join(data) %>%
        group_by(Training.file) %>%
        mutate(rel.error=pmin(max_rel_error, error/min.error)) %>%
        # ggplot(mapping=aes(eval(rlang::`!!`(attr)),factor(training.examples))) +
        ggplot(mapping=aes_(attr,~training.examples)) +
        geom_raster(aes(fill=rel.error)) +
        scale_fill_gradient(low = "white", high = "steelblue") +
        facet_wrap('Training.file', scales="free")
}

## ggsave(filename='hm-wordNgrams.png', plot=hmap(raw_data, wordNgrams), dpi=50, width=5, height=5, scale=2)
## ggsave(filename='hm-lr.png', plot=hmap(raw_data, lr), dpi=50, width=5, height=5, scale=2)
## ggsave(filename='hm-dim.png', plot=hmap(raw_data, dim), dpi=50, width=5, height=5, scale=2)
## ggsave(filename='hm-loss.png', plot=hmap(raw_data, loss), dpi=50, width=5, height=5, scale=2)
